/*
 * Copyright (c) 2001-2003 Swedish Institute of Computer Science.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

#include "arch/perf.h"

#include <stdio.h>
#include <stdlib.h>

/* For pbms_perf */
static unsigned long __c1l, __c1h, __c2l, __c2h;

static FILE *f;
static FILE *f1;
static FILE *f2;
static FILE *f3;

static void
perf_print_anon(unsigned long c1l, unsigned long c1h,
	   unsigned long c2l, unsigned long c2h){
  unsigned long sub_ms, sub_ls;

  sub_ms = c2h - c1h;
  sub_ls = c2l - c1l;
  if (c2l < c1l) sub_ms--;

  fprintf(f1, "%.8lu%.8lu ", sub_ms, sub_ls);
  fflush(NULL);  
}

static void
perf_print_anon2(unsigned long c1l, unsigned long c1h,
	   unsigned long c2l, unsigned long c2h){
  unsigned long long timestamp = 0;
  unsigned long long temp_higher = 0;
  timestamp |= (c2l - c1l);
  temp_higher = (c2h - c1h);
  timestamp |= (temp_higher << 32);

  fprintf(f2, "%.16llu\n", timestamp);
  fflush(NULL);  
}

static void
perf_print_file(unsigned long c1l, unsigned long c1h,
	   unsigned long c2l, unsigned long c2h, FILE *fname){
  unsigned long long timestamp = 0;
  unsigned long long temp_higher = 0;
  timestamp |= (c2l - c1l);
  temp_higher = (c2h - c1h);
  timestamp |= (temp_higher << 32);

  fprintf(fname, "%.16llu\n", timestamp);
  fflush(NULL);  
}


void
perf_print_start()
{
  struct timeval tv;
  gettimeofday(&tv,NULL);
  fprintf(f, "%lu,%06lu\n", tv.tv_sec, tv.tv_usec);
/*  fflush(NULL);  */
}

void
perf_print_start2()
{
  struct timeval tv;
  gettimeofday(&tv,NULL);
  fprintf(f2, "%lu,%06lu\n", tv.tv_sec, tv.tv_usec);
}

void
perf_init(char *fname)
{
  struct timeval tv;
  gettimeofday(&tv,NULL);
  printf("Opened debug performance file. Gettimeofday :: Sec: %ld, Usec: %ld\n",tv.tv_sec,tv.tv_usec);
  f = fopen(fname, "w");  
  f3 = fopen("dummy3", "w");  
}

void
perf_init2(char *fname)
{
  struct timeval tv;
  gettimeofday(&tv,NULL);
  printf("Opened 2nd debug performance file. Gettimeofday :: Sec: %ld, Usec: %ld\n",tv.tv_sec,tv.tv_usec);
  f2 = fopen(fname, "w");  
}	

void
perf_init_all(char *fname1, char *fname2, char *fname3)
{
  f1 = fopen(fname1, "w");  
  if(f1 == NULL){
    perror("fopen");
    exit(-1);
  }
  f2 = fopen(fname2, "w");  
  if(f2 == NULL){
    perror("fopen");
    exit(-1);
  }
  f3 = fopen(fname3, "w");    
    if(f2 == NULL){
    perror("fopen");
    exit(-1);
  }
  printf("Opened 3 debug performance files.\n");
}	

void pbms_perf_start(){
  __asm__(".byte 0x0f, 0x31" : "=a" (__c1l), "=d" (__c1h));
}

void pbms_perf_stop( char *key){
  __asm__(".byte 0x0f, 0x31" : "=a" (__c2l), "=d" (__c2h));
  /*perf_print_anon(__c1l, __c1h, __c2l, __c2h);*/
  perf_print_anon2(__c1l, __c1h, __c2l, __c2h);
}

void pbms_perf_start_serialized(){
  __asm__ __volatile__ (     
     "xorl %%eax,%%eax \n        cpuid"
     ::: "%rax", "%rbx", "%rcx", "%rdx");
     
  __asm__(".byte 0x0f, 0x31" : "=a" (__c1l), "=d" (__c1h));
}

void pbms_perf_stop_serialized(){
  __asm__ __volatile__ (     
     "xorl %%eax,%%eax \n        cpuid"
     ::: "%rax", "%rbx", "%rcx", "%rdx");
     
  __asm__(".byte 0x0f, 0x31" : "=a" (__c2l), "=d" (__c2h));
  perf_print_file(__c1l, __c1h, __c2l, __c2h, f3);
}

/* 
 * original lwIP functions below 
 */
 
void
perf_print(unsigned long c1l, unsigned long c1h,
	   unsigned long c2l, unsigned long c2h,
	   char *key)
{
  unsigned long sub_ms, sub_ls;
  
  sub_ms = c2h - c1h;
  sub_ls = c2l - c1l;
  if (c2l < c1l) sub_ms--;
  fprintf(f3, "%s: %.8lu%.8lu\n", key, sub_ms, sub_ls);
  fflush(NULL);
}
 
void
perf_print_times(struct tms *start, struct tms *end, char *key)
{
  fprintf(f, "%s: %lu\n", key, end->tms_stime - start->tms_stime);
  fflush(NULL);  
}
 

