#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <string.h>

#include "lwip/opt.h"
#include "lwip/init.h"

#include "lwip/mem.h"
#include "lwip/memp.h"
#include "lwip/sys.h"

#include "lwip/stats.h"

#include "lwip/inet_chksum.h"
#include "lwip/tcpip.h"
#include "lwip/sockets.h"

#include "netif/tapif.h"
#include "netif/loopif.h"
#include "netif/tcpdump.h"

#include "lwip/ip_addr.h"

#include "arch/perf.h"
#include "arch/sys_arch.h"

#define NUM_THREADS 1
#define SOCKETS_PER_THREAD 5
#define EVER ;;

#define PERF_LOOPIF 0
#define PERF_TAPIF (PERF_LOOPIF==0)

#if PERF_LOOPIF
#ifndef LWIP_HAVE_LOOPIF
#error "Must use loopback interface."
#endif
#endif /*PERF_LOOPIF*/

char name[] = "PBMS select";
char filename[] = "dummy.csv";
char filename2[] = "dummy2.csv";
#define INP_NAME_SIZE 100
char inputname[INP_NAME_SIZE];
char inputname2[INP_NAME_SIZE];
char perror_string[100];

/* (manual) host IP configuration */
static struct ip_addr ipaddr, netmask, gw;

/* nonstatic debug cmd option, exported in lwipopts.h */
unsigned char debug_flags;

#if PERF_LOOPIF
struct netif loopif;
#endif

#if PERF_TAPIF
struct netif tapif;
#endif

struct priority_arg {
  int socket;
  u16_t send_port;
  int send_tos;
  u16_t recv_port;
  sys_sem_t sem;
  int packets;
  u16_t own_port;
};  

struct thread_arg {
  int sockets[SOCKETS_PER_THREAD];
  u16_t send_ports[SOCKETS_PER_THREAD];
  u16_t recv_ports[SOCKETS_PER_THREAD];
  sys_sem_t sem;
  int packets;
};  

struct big {
  int numbers[350];
  int send_value;
};

static void
tcpip_init_done(void *arg)
{
  sys_sem_t *sem;
  sem = arg;
  sys_sem_signal(*sem);
}

/*
 *
 * SOCKET THREAD
 * Selective receive of replies to priority_thread's transmitted messages
 */
static void
socket_thread(void *arg)
{
  int i;
  struct sockaddr_in remote_addr;
  struct sockaddr_in my_addr;

  /* for socket receive */
  struct big big_send;
  int rx = 0;
  socklen_t addr_size = 0;
  int ioctl_arg = 0;

  /* for getsockopt */
  int optval = 0;
  socklen_t optlen; 

   /* for select */
  struct timeval stv;
  fd_set master; 
  fd_set read_fds;
  int fd_max;     
  int select_return;

  struct thread_arg * ta;
  ta = arg;

  /* REMOTE ADDRESS */
  remote_addr.sin_family = AF_INET;
  remote_addr.sin_addr.s_addr = inet_addr("129.241.187.94");
  memset(remote_addr.sin_zero,'\0',sizeof remote_addr.sin_zero);

  /* MY_ADDR FOR BIND() */
  my_addr.sin_family = AF_INET;
  my_addr.sin_addr.s_addr = inet_addr("192.168.0.2");
  my_addr.sin_addr.s_addr = inet_addr("127.0.0.1");
  memset(my_addr.sin_zero,'\0',sizeof my_addr.sin_zero);

  /* SOCKET(), BIND() */
  /* Nonblocking receive */
  ioctl_arg = 1;       
  for(i=0; i<SOCKETS_PER_THREAD; i++){
    ta->sockets[i] = lwip_socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if(ta->sockets[i] == -1){
      sprintf(perror_string,"sockets[%d]: socket",ta->sockets[i]);
      perror(perror_string);
      return;
    }

    my_addr.sin_port = htons(ta->send_ports[i]);

    if( lwip_bind(ta->sockets[i], (struct sockaddr *) &my_addr, (socklen_t) sizeof(my_addr) ) == -1 ){
      sprintf(perror_string,"sockets[%d]: bind",ta->sockets[i]);  
      perror(perror_string);      
      return;
    }

    lwip_ioctl(ta->sockets[i],FIONBIO,&ioctl_arg);     
  }

  addr_size = sizeof remote_addr;

  /* Signal main thread that init is complete */
  sys_sem_signal(ta->sem);
  sys_sem_wait(ta->sem);

  /* SELECT() */
  stv.tv_sec = 10;
  stv.tv_usec = 0;  
  FD_ZERO(&read_fds);
  FD_ZERO(&master);

	for(i=0; i<SOCKETS_PER_THREAD ; i++){
	  FD_SET(ta->sockets[i],&master);
    fd_max = (ta->sockets[i]>fd_max) ? ta->sockets[i] : fd_max;
	}

  for(EVER){
    read_fds = master; 
    select_return = select(fd_max+1, &read_fds, NULL, NULL, &stv);
    if (select_return == -1) {
      perror("select");
      exit(1);
    }
    else if(select_return == 0){
      printf("Select timeout\n");
      break;
    }
    else {
      perf_print_start2();
    }

    /* run through the existing connections looking for data to read */
    for(i = 0; i <= fd_max; i++) {
      if (FD_ISSET(i, &read_fds)) { 
        if(recvfrom(i, &big_send, sizeof(big_send) , 0, (struct sockaddr *)&remote_addr, &addr_size) <= 0){
          perror("recv");
          FD_CLR(i, &master);
        }
        else {
          rx++;
        } 

      } /* if FD_ISSET(i, &read_fds) */

    } /* for i in fd_max */

  } /* for EVER */

  /* ASSERT NO ERRORS OCCURED */
  optlen = sizeof optval;
  for(i=0; i<SOCKETS_PER_THREAD; i++){
    if( lwip_getsockopt(ta->sockets[i], SOL_SOCKET, SO_ERROR, &optval, &optlen ) == -1){
      sprintf(perror_string,"sockets[%d]: getsockopt",ta->sockets[i]);
      perror(perror_string);
      return;
    }
    LWIP_ASSERT("PBMS: No errors in socket after test run", (optval == ERR_OK) );

    if( lwip_close(ta->sockets[i]) == -1 ){
      sprintf(perror_string,"sockets[%d]: close",ta->sockets[i]);
      perror(perror_string);
      return;
    }
  }
  printf("%s: Sockets %d-%d selective receive finished. RX: %d.\n", name, ta->sockets[0], ta->sockets[SOCKETS_PER_THREAD-1], rx);
} 

/*
 *
 * PRIORITY THREAD
 * Send packets only, socket_thread will do selective receive of replies.
 */
static void
priority_thread(void *arg)
{
  int i;
  struct sockaddr_in remote_addr;
  struct sockaddr_in my_addr;

  /* for socket send */
  struct big big_send;
  int send_count = 0;
  unsigned int send_flags = 0;
  int tx = 0;

  /* for getsockopt */
  int optval = 0;
  socklen_t optlen;   

  struct priority_arg * ta;
  ta = arg;
  
  /* SOCKET() */
  ta->socket = lwip_socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  if(ta->socket == -1){
    perror("Priority thread: socket");
    return;
  }
    
  /* BIND() */
  my_addr.sin_family = AF_INET;
  my_addr.sin_port = htons(ta->send_port);
  my_addr.sin_addr.s_addr = inet_addr("192.168.0.2");
  memset(my_addr.sin_zero,'\0',sizeof my_addr.sin_zero);
  if( lwip_bind(ta->socket, (struct sockaddr *) &my_addr, (socklen_t) sizeof(my_addr) ) == -1 ){
    perror("Priority thread: bind");
    return;
  }

  /* SETSOCKOPT() - TOS */
  if( lwip_setsockopt(ta->socket, IPPROTO_IP, IP_TOS, &(ta->send_tos), (socklen_t) sizeof(ta->send_tos) ) == -1){
    perror("Priority thread: setsockopt");
    return;
  }  

  /* REMOTE ADDRESS */
  remote_addr.sin_family = AF_INET;
  remote_addr.sin_port = htons(ta->recv_port);
  remote_addr.sin_addr.s_addr = inet_addr("129.241.187.94");
  memset(remote_addr.sin_zero,'\0',sizeof remote_addr.sin_zero);

  /* Signal main thread that init is complete */
  sys_sem_signal(ta->sem);
  sys_sem_wait(ta->sem);

  for(i = 0; i < ta->packets ; i++ ){
    big_send.send_value = i;
    send_count = lwip_sendto(ta->socket, &big_send, sizeof(big_send), send_flags, 
                              (struct sockaddr*)&remote_addr, (socklen_t) sizeof(remote_addr) );

    if( send_count == -EWOULDBLOCK){
      printf("%s: Socket%d EWOULDBLOCK, breaking send loop\n", name, ta->socket);        
      break;
    }
    else if(send_count == -1){
      sprintf(perror_string,"Priority thread: sendto %d",i);
      perror(perror_string);
      return;
    } 
    else {
      tx++;
    }
  }

  /* ASSERT NO ERRORS OCCURED */
  optlen = sizeof(optval);
  if( lwip_getsockopt(ta->socket, SOL_SOCKET, SO_ERROR, &optval, &optlen ) == -1){
    perror("Priority thread: getsockopt");
    return;
  }
  LWIP_ASSERT("PBMS: No errors in socket after test run", (optval == ERR_OK) );

  if( lwip_close(ta->socket)==-1 ){
    perror("priority socket close");
    return;
  }

  printf("%s: Priority socket%d finished. Transmitted %d packets.\n", name, ta->socket, tx);
} 

void test_mbox_capacity(){
  sys_mbox_t mbox;
  int mbox_posts = 0;
  int netbuf_allocs = 0;
  int free_counter = 0;
  err_t post_return;
  struct netbuf *buf = NULL;

  mbox = sys_mbox_new(TCPIP_MBOX_SIZE);
  do {
    buf = memp_malloc(MEMP_NETBUF);
    if (buf == NULL) {
      printf("%s :: netbuf allocation error after %d allocations\n", name, netbuf_allocs);
      break;
    } 
    else {
      netbuf_allocs++;
    }

    post_return = sys_mbox_trypost(mbox, buf);  
    if( post_return == ERR_OK ){
      mbox_posts++;
    }
    else {
      memp_free(MEMP_NETBUF,(void *)buf);
      LWIP_ASSERT("PBMS: message box capacity as expected", (mbox_posts == (SYS_MBOX_SIZE-1)) );
      LWIP_ASSERT("PBMS: trypost returned ERR_MEM", (post_return == ERR_MEM) );
      printf("%s: mbox test run: mbox full after %d posts \n",name ,mbox_posts );      
      break;
    } 
  } while(1);

  printf("%s: mbox test run: freed NETBUF ",name );
  for(free_counter = 0; free_counter < mbox_posts; free_counter++){
    sys_mbox_fetch(mbox,(void *)&buf);
    memp_free(MEMP_NETBUF,(void *)buf);
    printf("#%d, ",free_counter);
  }
  printf("mbox test run done. \n");
}

void init(){
  sys_sem_t sem;
  struct in_addr inaddr;
  char ip_str[16] = {0}, nm_str[16] = {0}, gw_str[16] = {0};
  
  /* use debug flags defined by debug.h */
  debug_flags |= (LWIP_DBG_ON|LWIP_DBG_TRACE|LWIP_DBG_STATE|LWIP_DBG_FRESH|LWIP_DBG_HALT);

  lwip_init();
  stats_init();
 
  netif_init();
  sem = sys_sem_new(0);
  tcpip_init(tcpip_init_done, &sem);
  sys_sem_wait(sem);
  sys_sem_free(sem);

#if PERF_TAPIF
  IP4_ADDR(&gw, 192,168,0,1);
  IP4_ADDR(&netmask, 255,255,255,0);
  IP4_ADDR(&ipaddr, 192,168,0,2);

  printf("%s :: Using tap interface.\n", name);
  netif_set_default(netif_add(&tapif,&ipaddr, &netmask, &gw, NULL, tapif_init,
                  tcpip_input));
  netif_set_up(&tapif);
#endif

#if PERF_LOOPIF
  IP4_ADDR(&gw, 127,0,0,1);
  IP4_ADDR(&ipaddr, 127,0,0,1);
  IP4_ADDR(&netmask, 255,0,0,0);

  printf("%s :: Using loopback interface.\n", name);
  netif_set_default(netif_add(&loopif, &ipaddr, &netmask, &gw, NULL, loopif_init,
        tcpip_input));
  netif_set_up(&loopif);        
#endif

  inaddr.s_addr = ipaddr.addr;
  strncpy(ip_str,inet_ntoa(inaddr),sizeof(ip_str));
  inaddr.s_addr = netmask.addr;
  strncpy(nm_str,inet_ntoa(inaddr),sizeof(nm_str));
  inaddr.s_addr = gw.addr;
  strncpy(gw_str,inet_ntoa(inaddr),sizeof(gw_str));
  printf("Host at %s mask %s gateway %s\n", ip_str, nm_str, gw_str);

  printf("%s :: System initialized.\n", name);
  
  test_mbox_capacity();
}

int
main(int argc, char **argv)
{
  int i,j;
  const int packets = 40000;
  const int pri_packets = 10;
  int packets_per_socket = packets /(NUM_THREADS * SOCKETS_PER_THREAD);

  struct thread_arg threads[NUM_THREADS];
  struct priority_arg pri_arg;

  system("clear");
  system("xset b on");
  
  memset(inputname,'\0',INP_NAME_SIZE);
  if (argc == 1){
    printf("Optional command line argument: Debug print file name.\n");    
    printf("Using default name: %s\n",filename);
  }
  else if (argc == 3) {
    strncpy(inputname, argv[1], INP_NAME_SIZE-1);
    strncpy(inputname2, argv[2], INP_NAME_SIZE-1);
    printf("Debug perf print file 1 name: %s, file 2 name: %s\n",inputname, inputname2);
  }
  else {
    printf("Command line argument: Optional two file names only. Exiting.\n");
    exit(0);
  }

  pri_arg.socket = -1;
  pri_arg.send_port = 10000;
  pri_arg.send_tos = IPTOS_PREC_PRIORITY;
  pri_arg.recv_port = 20000;
  pri_arg.sem = sys_sem_new(0);
  pri_arg.packets = pri_packets;

  for(i=0; i<NUM_THREADS ; i++ ){
    for(j=0; j<SOCKETS_PER_THREAD ; j++ ){
      threads[i].sockets[j] = -1;
      threads[i].send_ports[j] = 10001 + i;
      threads[i].recv_ports[j] = 20001 + j;  
    }
    threads[i].sem = sys_sem_new(0);
    threads[i].packets = packets_per_socket;
  }

  init(); 

  #ifdef PERF
  if(argc==1){
    perf_init(filename);
    perf_init2(filename2);    
  }
  else if (argc==3) {
    perf_init(inputname);  
    perf_init2(inputname2);        
  }
  else {
    printf("perf init error. Exit.\n");
    exit(0);
  }
  #endif /* PERF */

  sleep(10);

  /* Thread initialization */
  sys_thread_new("priority_thread", priority_thread, (void *)&pri_arg, DEFAULT_THREAD_STACKSIZE, DEFAULT_THREAD_PRIO);
  printf("%s: Main: Waiting for priority thread init.\n", name);
  sys_sem_wait(pri_arg.sem);

  printf("%s: Main: Waiting for thread inits:", name);
  for(i=0; i<NUM_THREADS ; i++ ){
    sys_thread_new("socket_thread", socket_thread, (void *)&threads[i], DEFAULT_THREAD_STACKSIZE, DEFAULT_THREAD_PRIO);
    printf("%d ",i);
    sys_sem_wait(threads[i].sem);
  }
  printf(" :: All done.\n");

  /* TX START SYNCHRONIZATION */
  printf("%s: Main: Routine threads begin RX: ", name);
  for(i=0; i<NUM_THREADS ; i++ ){
    printf("%d ",i);
    sys_sem_signal(threads[i].sem);
  }
  printf(" :: All started.\n");
  
  printf("%s: Main: Signalling priority thread to begin transmission.\n", name);
  sys_sem_signal(pri_arg.sem);  

  sleep(20);

  /* TODO: TX END SYNCHRONIZATION ? */

  /*stats_display();*/
  printf("\a");
  return 0;
}


