#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <string.h>

#include "lwip/opt.h"
#include "lwip/init.h"

#include "lwip/mem.h"
#include "lwip/memp.h"
#include "lwip/sys.h"

#include "lwip/stats.h"

#include "lwip/inet_chksum.h"
#include "lwip/tcpip.h"
#include "lwip/sockets.h"

#include "netif/tapif.h"
#include "netif/loopif.h"
#include "netif/tcpdump.h"

#include "lwip/ip_addr.h"
#include "lwip/debug.h"
#include "arch/perf.h"
#include "arch/sys_arch.h"

#ifndef LWIP_HAVE_LOOPIF
#error "Must use loopback interface."
#endif

char name[] = "TEST 3";

/* (manual) host IP configuration */
static struct ip_addr ipaddr, netmask, gw;

/* nonstatic debug cmd option, exported in lwipopts.h */
unsigned char debug_flags;

static void
tcpip_init_done(void *arg)
{
  sys_sem_t *sem;
  sem = arg;
  sys_sem_signal(*sem);
}

struct netif loopif;

static void
main_thread(void *arg)
{
  int iterations = 2;
  int iter_counter = 0; 

  sys_sem_t sem;
  
  const int packets = 10;

  int socket1 = -1;
  const u16_t s1_port = 10000;
  struct sockaddr_in s1_addr;
  int optval;   
  socklen_t optlen;

  int ioctl_arg = 0;
  
  int socket2 = -1;
  const u16_t s2_port = 20000;
  struct sockaddr_in s2_addr;
  
  int send_count;
  unsigned int send_flags;
  struct sockaddr_in dest_addr;
  socklen_t dest_len;
  int send_i = 0;
  int send_value = 0;

  int recv_count;
  int recv_value;
  unsigned int recv_flags;
  struct sockaddr send_addr;
  socklen_t send_addr_size;

  netif_init();
  sem = sys_sem_new(0);
  tcpip_init(tcpip_init_done, &sem);
  sys_sem_wait(sem);
  sys_sem_free(sem);
  printf("TCP/IP initialized.\n");
 
  IP4_ADDR(&gw, 127,0,0,1);
  IP4_ADDR(&ipaddr, 127,0,0,1);
  IP4_ADDR(&netmask, 255,0,0,0);
  netif_set_default(netif_add(&loopif, &ipaddr, &netmask, &gw, NULL, loopif_init,
        tcpip_input));
  netif_set_up(&loopif);        

  /* SOCKET() */
  socket1 = lwip_socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  if(socket1 == -1){
    perror("socket1");
  }

  /* BIND() */
  s1_addr.sin_family = AF_INET;
  s1_addr.sin_port = htons(s1_port);
  s1_addr.sin_addr.s_addr = inet_addr("127.0.0.1");
  memset(s1_addr.sin_zero,'\0',sizeof s1_addr.sin_zero);
  if( lwip_bind(socket1, (struct sockaddr *) &s1_addr, (socklen_t) sizeof(s1_addr) ) == -1 ){
    perror("bind");  
  }

  /* GETSOCKOPT() */
  optlen = sizeof(optval);

  /* SO_RCVBUF */
  if( lwip_getsockopt(socket1, SOL_SOCKET, SO_RCVBUF, &optval, &optlen ) == -1){
    perror("getsockopt");
  }
  LWIP_DEBUGF(SERVER_DEBUG, ("%s: Socket%d SO_RCVBUF: %d \n", name, socket1 , optval) );

  /* SO_ERROR */
  if( lwip_getsockopt(socket1, SOL_SOCKET, SO_ERROR, &optval, &optlen ) == -1){
    perror("getsockopt");
  }
  LWIP_DEBUGF(SERVER_DEBUG, ("%s: Socket%d SO_ERROR: %d \n", name, socket1 , optval) );

  /* SO_NO_CHECK */
  if( lwip_getsockopt(socket1, SOL_SOCKET, SO_NO_CHECK, &optval, &optlen ) == -1){
    perror("getsockopt");
  }
  LWIP_DEBUGF(SERVER_DEBUG, ("%s: Socket%d SO_NO_CHECK: %d \n", name, socket1 , optval) );
  
  /* SOCKET 2 */
  socket2 = lwip_socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);  
  if(socket2 == -1){
    perror("socket2");
  }
  s2_addr.sin_family = AF_INET;
  s2_addr.sin_port = htons(s2_port);
  s2_addr.sin_addr.s_addr = inet_addr("127.0.0.1");
  memset(s2_addr.sin_zero,'\0',sizeof s2_addr.sin_zero);
  if( lwip_bind(socket2, (struct sockaddr *) &s2_addr, (socklen_t) sizeof(s2_addr) ) == -1 ){
    perror("bind");  
  }

  /* IOCTL */
  ioctl_arg = 1;   
  lwip_ioctl(socket2,FIONBIO,&ioctl_arg);

  for(iter_counter = 0; iter_counter < iterations; iter_counter++ ){

    if(iter_counter == 1 ){
      /* test MSG_DONTWAIT */
      ioctl_arg = 0;   
      lwip_ioctl(socket2,FIONBIO,&ioctl_arg);      
    }

    /* SEND FROM SOCKET1 TO SOCKET2 */
    send_flags = 0;
    dest_addr.sin_len = sizeof(dest_addr);
    memset(dest_addr.sin_zero,'\0',sizeof dest_addr.sin_zero);
    dest_addr.sin_port = htons(s2_port);
    dest_addr.sin_family = AF_INET;
    dest_addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
    /*dest_addr.sin_addr.s_addr = inet_addr("127.0.0.1");*/
    dest_len = sizeof(dest_addr);

    LWIP_DEBUGF(SERVER_DEBUG, ("%s: Send loop: (send series number, send number, socket number, sent value, sizeof value): \n", name) );
    LWIP_DEBUGF(SERVER_DEBUG, ("\t") );
    for(send_i = 0; send_i<packets ; send_i++){
      send_value = send_i + 10*iter_counter;
      send_count = lwip_sendto(socket1, &send_value, sizeof(send_value), send_flags, (struct sockaddr*)&dest_addr, dest_len );
      if( send_count == -1){
        perror("sendto");
      }
      LWIP_DEBUGF(SERVER_DEBUG, ("(%d,%d,%d,%d,%d) ", iter_counter, send_i, socket1 , send_value, send_count) );
    }
    LWIP_DEBUGF(SERVER_DEBUG, ("\n\n") );

    /* Hypothesis: value "left hanging" in test 1 was due to speedy transmitting */
    LWIP_DEBUGF(SERVER_DEBUG, ("%s: Thread sleeping to let all sends be processed.\n",name));
    sleep(1);

    recv_flags = (iter_counter == 1) ? MSG_DONTWAIT:0;
    send_addr_size = sizeof(send_addr);

    LWIP_DEBUGF(SERVER_DEBUG, ("%s: Receive loop: (receive series number, receive number, socket number, received value, sizeof value): \n", name) );
    LWIP_DEBUGF(SERVER_DEBUG, ("\t") );
    for(send_i = 0; send_i<packets ; send_i++){
      recv_count = lwip_recvfrom(socket2, (void *) &recv_value, sizeof(recv_value), recv_flags, &send_addr, &send_addr_size );
      if( recv_count == -1 ){
        /* SO_ERROR */
        if( lwip_getsockopt(socket2, SOL_SOCKET, SO_ERROR, &optval, &optlen ) == -1){
          perror("getsockopt");
        }
        if(optval == EWOULDBLOCK){
          LWIP_DEBUGF(SERVER_DEBUG, ("\n%s: Socket%d EWOULDBLOCK, breaking recv loop", name, socket2) );        
          break;
        }
      }
      else {
        LWIP_DEBUGF(SERVER_DEBUG, ("(%d,%d,%d,%d,%d) ", iter_counter, send_i, socket2 , recv_value, recv_count) );    
      }
    }
    LWIP_DEBUGF(SERVER_DEBUG, ("\n\n") );    

    
    /* SO_ERROR */
    if( lwip_getsockopt(socket1, SOL_SOCKET, SO_ERROR, &optval, &optlen ) == -1){
      perror("getsockopt");
    }
    LWIP_DEBUGF(SERVER_DEBUG, ("%s: Socket%d SO_ERROR: %d \n", name, socket1 , optval) );

    /* SO_ERROR */
    if( lwip_getsockopt(socket2, SOL_SOCKET, SO_ERROR, &optval, &optlen ) == -1){
      perror("getsockopt");
    }
    LWIP_DEBUGF(SERVER_DEBUG, ("%s: Socket%d SO_ERROR: %d \n", name,socket2 , optval) );

  } /* end for(iterations) */
  LWIP_DEBUGF(SERVER_DEBUG, ("%s: Thread finished, test completed.\n", name) );
/*  exit(0);*/
}

int
main(int argc, char **argv)
{
  const int sleeptime = 10; 
  sys_mbox_t mbox;
  int mbox_posts = 0;
  int netbuf_allocs = 0;
  err_t post_return;
  struct netbuf *buf = NULL;
 
  /* use debug flags defined by debug.h */
  debug_flags |= (LWIP_DBG_ON|LWIP_DBG_TRACE|LWIP_DBG_STATE|LWIP_DBG_FRESH|LWIP_DBG_HALT);
  lwip_init();
  stats_init();
  system("clear");
  printf("%s :: System initialized.\n", name);

  /* Test everything mbox-related */
  mbox = sys_mbox_new(TCPIP_MBOX_SIZE);

  do {
    buf = memp_malloc(MEMP_NETBUF);
    if (buf == NULL) {
      printf("%s :: netbuf allocation error after %d allocations\n", name, netbuf_allocs);
      break;
    } 
    else {
      netbuf_allocs++;
    }

    post_return = sys_mbox_trypost(mbox, buf);  
    if( post_return == ERR_OK ){
      mbox_posts++;
    }
    else {
      printf("%s :: mbox full after %d posts \n",name ,mbox_posts );      
      printf("%s :: trypost returned ERR_MEM? %s \n",name ,  (post_return == ERR_MEM) ? "true":"false" );
      break;
    } 
  } while(1);

  sys_thread_new("main_thread", main_thread, NULL, DEFAULT_THREAD_STACKSIZE, DEFAULT_THREAD_PRIO);
  LWIP_DEBUGF(SERVER_DEBUG, ("%s: Main: Test will timeout after %d seconds.\n\n", name, sleeptime) );
  sleep(sleeptime);
  LWIP_DEBUGF(SERVER_DEBUG, ("%s: Main: Test timeout.\n", name) );

  stats_display();
  return 0;
}


