#ifndef PRIORITIES_H
#define PRIORITIES_H

/* from sockets.h
#define IPTOS_PREC_NETCONTROL           0xe0, 0b 1110 0000
#define IPTOS_PREC_INTERNETCONTROL      0xc0, 0b 1100 0000
#define IPTOS_PREC_CRITIC_ECP           0xa0, 0b 1010 0000
#define IPTOS_PREC_FLASHOVERRIDE        0x80, 0b 1000 0000
#define IPTOS_PREC_FLASH                0x60, 0b 0110 0000
#define IPTOS_PREC_IMMEDIATE            0x40, 0b 0110 0000
#define IPTOS_PREC_PRIORITY             0x20, 0b 0010 0000
#define IPTOS_PREC_ROUTINE              0x00, 0b 0000 0000

Convert to decimal: shift right 5 times

*/

#define STRICT_PRI_Q 1
#define DEBUG_PRI_Q 1

/* The range of assigned priorities will be [0, PBMS_PRIORITY_LEVELS-1] */
#define PBMS_PRIORITY_LEVELS 3
#define PBMS_DEFAULT_PRIORITY 1

/* SO_RETURN_EWB
 * Using this option nonblocking sockets do not 
 * get EWOULDBLOCK as an error condition, 
 * but merely return that value upon completion of sendto or recvfrom
 */

#define SO_RETURN_EWB 1

/* tcpip_input() */
#define PBMS_INPUT_PREC_NETCONTROL        2
#define PBMS_INPUT_PREC_INTERNETCONTROL   2
#define PBMS_INPUT_PREC_CRITIC_ECP        2
#define PBMS_INPUT_PREC_FLASHOVERRIDE     2
#define PBMS_INPUT_PREC_FLASH             2
#define PBMS_INPUT_PREC_IMMEDIATE         2
#define PBMS_INPUT_PREC_PRIORITY          2
#define PBMS_INPUT_PREC_ROUTINE           0

/* tcpip_callback_with_block() */
#define PBMS_CALLBACK_PRIORITY    PBMS_DEFAULT_PRIORITY

/* tcpip_timeout() */
#define PBMS_TIMEOUT_PRIORITY     PBMS_DEFAULT_PRIORITY

/* tcpip_apimsg() */
#define PBMS_NEWCONN_PRIORITY     PBMS_DEFAULT_PRIORITY
#define PBMS_DELCONN_PRIORITY     PBMS_DEFAULT_PRIORITY
#define PBMS_BIND_PRIORITY        PBMS_DEFAULT_PRIORITY
#define PBMS_CONNECT_PRIORITY     PBMS_DEFAULT_PRIORITY
#define PBMS_DISCONNECT_PRIORITY  PBMS_DEFAULT_PRIORITY
/* more than one priority level for send, see below */
/*#define PBMS_SEND_PRIORITY        0*/

/* more than one priority level for recv, see below */
/*#define PBMS_RECV_PRIORITY        0*/
#define PBMS_GETADDR_PRIORITY     PBMS_DEFAULT_PRIORITY

/* tcpip_apimsg() (calling do_send) */
#define PBMS_OUTPUT_PREC_NETCONTROL        2
#define PBMS_OUTPUT_PREC_INTERNETCONTROL   2
#define PBMS_OUTPUT_PREC_CRITIC_ECP        2
#define PBMS_OUTPUT_PREC_FLASHOVERRIDE     2
#define PBMS_OUTPUT_PREC_FLASH             2
#define PBMS_OUTPUT_PREC_IMMEDIATE         2
#define PBMS_OUTPUT_PREC_PRIORITY          2
#define PBMS_OUTPUT_PREC_ROUTINE           0

/* tcpip_apimsg() (calling do_recv) */
#define PBMS_RECV_PREC_NETCONTROL        2
#define PBMS_RECV_PREC_INTERNETCONTROL   2
#define PBMS_RECV_PREC_CRITIC_ECP        2
#define PBMS_RECV_PREC_FLASHOVERRIDE     2
#define PBMS_RECV_PREC_FLASH             2
#define PBMS_RECV_PREC_IMMEDIATE         2
#define PBMS_RECV_PREC_PRIORITY          2
#define PBMS_RECV_PREC_ROUTINE           0


/* TODO: Can ensure at compile time that no faulty priority scheme is allowed
#if (PBMS_PRIORITY_LEVELS<1)
  #error "PBMS: TOO FEW PRIORITY LEVELS"
#endif

#if ( PBMS_DELCONN_PRIORITY<0 ) || (PBMS_DELCONN_PRIORITY > (PBMS_PRIORITY_LEVELS-1) )
  #error "PBMS_DELCONN_PRIORITY is out of valid priority range"
#endif
#if ( PBMS_NEWCONN_PRIORITY<0 ) || (PBMS_NEWCONN_PRIORITY > (PBMS_PRIORITY_LEVELS-1) )
  #error "PBMS_NEWCONN_PRIORITY is out of valid priority range"
#endif
#if ( PBMS_INPUT_PRIORITY<0 ) || (PBMS_INPUT_PRIORITY > (PBMS_PRIORITY_LEVELS-1) )
  #error "PBMS_INPUT_PRIORITY is out of valid priority range"
#endif
#if ( PBMS_DEFAULT_PRIORITY<0 ) || (PBMS_DEFAULT_PRIORITY > (PBMS_PRIORITY_LEVELS-1) )
  #error "PBMS_DEFAULT_PRIORITY is out of valid priority range"
#endif
*/

#endif /*  PRIORITIES_H */

