#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <string.h>

#include "lwip/opt.h"
#include "lwip/init.h"

#include "lwip/mem.h"
#include "lwip/memp.h"
#include "lwip/sys.h"

#include "lwip/stats.h"

#include "lwip/inet_chksum.h"
#include "lwip/tcpip.h"
#include "lwip/sockets.h"

#include "netif/tapif.h"
#include "netif/loopif.h"
#include "netif/tcpdump.h"

#include "lwip/ip_addr.h"
#include "lwip/debug.h"
#include "arch/perf.h"
#include "arch/sys_arch.h"

#ifndef LWIP_HAVE_LOOPIF
#error "Must use loopback interface."
#endif

char name[] = "TEST 1";

/* (manual) host IP configuration */
static struct ip_addr ipaddr, netmask, gw;

/* nonstatic debug cmd option, exported in lwipopts.h */
unsigned char debug_flags;

static void
tcpip_init_done(void *arg)
{
  sys_sem_t *sem;
  sem = arg;
  sys_sem_signal(*sem);
}

struct netif loopif;

static void
main_thread(void *arg)
{
  sys_sem_t sem;
  
  const int packets = 10;

  int socket1 = -1;
  const u16_t s1_port = 10000;
  struct sockaddr_in s1_addr;

  int socket2 = -1;
  const u16_t s2_port = 20000;
  struct sockaddr_in s2_addr;
  
  int send_count;
  unsigned int send_flags;
  struct sockaddr_in dest_addr;
  socklen_t dest_len;
  int send_i = 0;

  int recv_count;
  int recv_value;
  unsigned int recv_flags;
  struct sockaddr send_addr;
  socklen_t send_addr_size;

  netif_init();
  sem = sys_sem_new(0);
  tcpip_init(tcpip_init_done, &sem);
  sys_sem_wait(sem);
  sys_sem_free(sem);
  printf("TCP/IP initialized.\n");
 
  IP4_ADDR(&gw, 127,0,0,1);
  IP4_ADDR(&ipaddr, 127,0,0,1);
  IP4_ADDR(&netmask, 255,0,0,0);
  netif_set_default(netif_add(&loopif, &ipaddr, &netmask, &gw, NULL, loopif_init,
        tcpip_input));
  netif_set_up(&loopif);        

  /* SOCKET() */
  socket1 = lwip_socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  if(socket1 == -1){
    perror("socket1");
  }

  /* BIND() */
  s1_addr.sin_family = AF_INET;
  s1_addr.sin_port = htons(s1_port);
  s1_addr.sin_addr.s_addr = inet_addr("127.0.0.1");
  memset(s1_addr.sin_zero,'\0',sizeof s1_addr.sin_zero);
  if( lwip_bind(socket1, (struct sockaddr *) &s1_addr, (socklen_t) sizeof(s1_addr) ) == -1 ){
    perror("bind");  
  }

  /* SOCKET 2 */
  socket2 = lwip_socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);  
  if(socket2 == -1){
    perror("socket2");
  }
  s2_addr.sin_family = AF_INET;
  s2_addr.sin_port = htons(s2_port);
  s2_addr.sin_addr.s_addr = inet_addr("127.0.0.1");
  memset(s2_addr.sin_zero,'\0',sizeof s2_addr.sin_zero);
  if( lwip_bind(socket2, (struct sockaddr *) &s2_addr, (socklen_t) sizeof(s2_addr) ) == -1 ){
    perror("bind");  
  }
    
  /* SEND FROM SOCKET1 TO SOCKET2 */
  send_flags = 0;
  dest_addr.sin_len = sizeof(dest_addr);
  memset(dest_addr.sin_zero,'\0',sizeof dest_addr.sin_zero);
  dest_addr.sin_port = htons(s2_port);
  dest_addr.sin_family = AF_INET;
  dest_addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
  /*dest_addr.sin_addr.s_addr = inet_addr("127.0.0.1");*/
  dest_len = sizeof(dest_addr);

  for(send_i = 0; send_i<packets ; send_i++){
    send_count = lwip_sendto(socket1, &send_i, sizeof(send_i), send_flags, (struct sockaddr*)&dest_addr, dest_len );
    if( send_count == -1){
      perror("sendto");
    }
    LWIP_DEBUGF(SERVER_DEBUG, ("Server: send #%d: Socket%d sent value: %d (size: %d bytes).\n", send_i, socket1 , send_i, send_count) );
  }

  recv_flags = 0;
  send_addr_size = sizeof(send_addr);
  for(send_i = 0; send_i<packets ; send_i++){
    recv_count = lwip_recvfrom(socket2, (void *) &recv_value, sizeof(recv_value), recv_flags, &send_addr, &send_addr_size );
    if( recv_count == -1 ){
      perror("recvfrom");
    }
    LWIP_DEBUGF(SERVER_DEBUG, ("Server: recv #%d: Socket%d received value: %d (size: %d bytes).\n",send_i, socket2 , recv_value, recv_count) );    
  }
  
  LWIP_DEBUGF(SERVER_DEBUG, ("Server: Thread finished\n") );
}

int
main(int argc, char **argv)
{
  int sleeptime = 1;
  /* use debug flags defined by debug.h */
  debug_flags |= (LWIP_DBG_ON|LWIP_DBG_TRACE|LWIP_DBG_STATE|LWIP_DBG_FRESH|LWIP_DBG_HALT);
  lwip_init();
  system("clear");
  printf("%s :: System initialized.\n", name);

  stats_init();
    
  sys_thread_new("main_thread", main_thread, NULL, DEFAULT_THREAD_STACKSIZE, DEFAULT_THREAD_PRIO);

  LWIP_DEBUGF(SERVER_DEBUG, ("\nServer main: sleep %d\n\n",sleeptime) );
  sleep(sleeptime);
  LWIP_DEBUGF(SERVER_DEBUG, ("\nServer main: wake\n\n") );
  
  stats_display();
  /*pause();*/
  return 0;
}


