/*
 * PBMS 2008
 */

#include "bootp.h"

/*
 * bootp_handler must be nonblocking, 
 * the entire stack management thread waits for it to complete.
 * delays here will not (?) affect the priority scheme, since 
 *
 * bootp requests are handled according to sender TOS, 
 * which will not be anything over ROUTINE?
 */
static err_t bootp_handler(struct pbuf *in_p, struct pbuf *out_p){
  return ERR_OK;
}

static void bootp_recv(void *arg, struct udp_pcb *pcb, struct pbuf *p, struct ip_addr *addr, u16_t port){
  printf("Hello World!\n");
/* pseudocode 
  struct pbuf *bootp_reply;
  (correct allocator?)  bootp_reply = mem_malloc(sizeof(struct pbuf));
  Allocate bootp_reply->payload ?
  (+error handling for both)
  
  
  if( bootp_handler(p,bootp_reply) != ERR_OK ){
    retry?
  }
  else {
    OK, 
    udp_sendto(pcb?,bootp_reply,addr?,port?);
  }
  
*/
  
/*  err_t
    udp_sendto(
      struct udp_pcb *pcb, 
      struct pbuf *p, 
      struct ip_addr *dst_ip, 
      u16_t dst_port
    ); */
  
}
err_t bootp_init(struct netif *netif){
  struct bootp *bootp;
  err_t result = ERR_OK;

  LWIP_ERROR("netif != NULL", (netif != NULL), return ERR_ARG;);
  bootp = netif->bootp;
  LWIP_DEBUGF(BOOTP_DEBUG | LWIP_DBG_TRACE | LWIP_DBG_STATE, ("bootp_init(netif=%p) %c%c%"U16_F"\n", (void*)netif, netif->name[0], netif->name[1], (u16_t)netif->num));

  /* Does PBMS BOOTP need a netif flag? Not for now. */

  LWIP_DEBUGF(BOOTP_DEBUG | LWIP_DBG_TRACE, ("bootp_init(): Allocating BOOTP struct\n"));
  bootp = mem_malloc(sizeof(struct bootp));
  if (bootp == NULL) {
    LWIP_DEBUGF(BOOTP_DEBUG | LWIP_DBG_TRACE, ("bootp_init(): could not allocate bootp\n"));
    return ERR_MEM;
  }
  /* store this bootp client in the netif */
  netif->bootp = bootp;
  LWIP_DEBUGF(BOOTP_DEBUG | LWIP_DBG_TRACE, ("bootp_init(): allocated bootp"));
    
  /* clear data structure */
  memset(bootp, 0, sizeof(struct bootp));

  /* allocate UDP PCB */
  bootp->pcb = udp_new();
  if (bootp->pcb == NULL) {
    LWIP_DEBUGF(BOOTP_DEBUG  | LWIP_DBG_TRACE, ("bootp_init(): could not obtain pcb\n"));
    mem_free((void *)bootp);
    netif->bootp = bootp = NULL;
    return ERR_MEM;
  }
  /* set up local and remote port for the pcb */
  udp_bind(bootp->pcb, IP_ADDR_ANY, BOOTP_SERVER_PORT);
  udp_connect(bootp->pcb, IP_ADDR_ANY, BOOTP_CLIENT_PORT);

  /* set up the recv callback and argument */
  udp_recv(bootp->pcb, bootp_recv, netif);
 
  return result;
}
